package PFARS;

##############################################################################
## PFARS.pm - PFARS For Automatic Replication System                        ##
##                                                                          ##
## Copyright (C) 2001, 2002 Paul J. Baker                                   ##
##                                                                          ##
## This program is free software; you can redistribute it and/or modify     ##
## it under the terms of the GNU General Public License as published by     ##
## the Free Software Foundation; either version 2 of the License, or        ##
## (at your option) any later version.                                      ##
##                                                                          ##
## This program is distributed in the hope that it will be useful,          ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
## GNU General Public License for more details.                             ##
##                                                                          ##
## You should have received a copy of the GNU General Public License        ##
## along with this program; if not, write to the Free Software              ##
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                ##
##                                                                          ##
## You may contact the author via electronic mail at pfars@paulbaker.net    ##
##############################################################################

## $Id: PFARS.pm,v 1.8 2002/07/18 11:26:09 pbaker Exp $

=head1 NAME

PFARS - PFARS For Automatic Replication System.

=head1 SYNOPSIS

  use PFARS;

=head1 DESCRIPTION

PFARS exports various functions used by B<PFARS> scripts.

=head1 EXPORT

Many by default.

=head1 AUTHOR

Paul Baker, pbaker@where2getit.com

=head1 SEE ALSO

pfars(8).

=head1 VERSION

0.9.2.2

=head1 FUNCTIONS

=cut

#############################################################################

$VERSION = '0.9.2.2';

#require 5.6.0;
use strict;

#use warnings::register;

use Carp;
use IO::File;
use Getopt::Long ();

require Exporter;

use vars qw( @ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION %option );

@ISA = qw( Exporter );

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
%EXPORT_TAGS = (
);

@EXPORT = qw( FATAL ERROR NORMAL VERBOSE DEBUG output );

@EXPORT_OK = @EXPORT, qw( %option );

use constant FATAL  => -1;
use constant ERROR  => 0;
use constant NORMAL => 1;
use constant VERBOSE  => 2;
use constant DEBUG  => 3;

## set option defaults.
%option = (
    auto    => 0,
    config  => '/etc/pfars/config',
    changes => 0,
    quiet   => 0,
    verbose => 0,
    debug   => 0,
    help    => 0,
    man     => 0,
    version => 0,
    noact   => 0,
);

my $output = 1;
my @output_cache = ();
my $change_state = 0;
my $watch_rsync = 0;

# prototypes.
sub output ($$@);


sub options
{
    my ($class, %args) = @_;
    
    Getopt::Long::GetOptions(
        'auto'          => \$PFARS::option{auto},
        'config|c=s'    => \$PFARS::option{config},
    
        'changes'       => \$PFARS::option{changes},
        'quiet|q'       => \$PFARS::option{quiet},
        'verbose|v+'    => \$PFARS::option{verbose},
        'debug|d'       => \$PFARS::option{debug},
        'no-act|n'      => \$PFARS::option{noact},
    
        'help|?'        => \$PFARS::option{help},
        'man'           => \$PFARS::option{man},
        'version|V'     => \$PFARS::option{version},
        %args
        
    ) or pod2usage(2);
    
    pod2usage(1) if $PFARS::option{help};
    pod2usage(-exitstatus => 0, -verbose => 2) if $PFARS::option{man};
    
    # display version number if --version set.
    if ($PFARS::option{version}) {
        print "PFARS For Automatic Replication System\n";
        print "Version $PFARS::VERSION\n";
        print q$Id: PFARS.pm,v 1.8 2002/07/18 11:26:09 pbaker Exp $, "\n";
        exit(0);
    }
}


sub output_init
{

    my ($class, $default) = @_;
    $output = defined $default ? $default : $output;
    
    $output = 0 if ($PFARS::option{quiet} and $PFARS::option{auto});
    $output += $PFARS::option{verbose};
    ## --debug supported for backward compatibility.
    $output = 3 if ($PFARS::option{debug} and $output < 3);

}

=head2 output

Handles output.

=cut

## FUNCTION output
##   Purpose: function used to display all output of the program.
##   Usage: output TYPE message
##     TYPE is either FATAL, ERROR, NORMAL, VERBOSE, or DEBUG.
##     message is the text to output.

sub output ($$@) {

    my $type = shift;

    my $state = PFARS->change_state;

    if ($type <= $output) {

        if ($PFARS::option{changes}) {

            PFARS->change_state('error') if $type < 1;

            push(@output_cache, @_);

            if ($state) {

                while (my $output = shift @output_cache) {

                    print $output;

                }

            }

        } else {

            print @_;

        }

        if ($type < 0) {

            # this was a fatal error
            exit(1);

        }

        return 1;
    }

    return 0;

} # sub output

## FUNCTION change_state
##   Purpose: function used to monitor if changes were made
##     during the replication.
##   Usage: change_state 'command'
##     Where command is 'rsync begin' to signify that an
##     rsync process has begun. Or 'rsync_done' to signify
##     that an rsync process has finished. Or 'error' meaning
##     an error has occured. Or 'force' to signify that a
##     change has been made.

sub change_state
{

    if ($change_state eq 'force') {

        return 1;

    }

    my ($class, $command) = @_;
    $command ||= 0;

    if ($command eq 'rsync begin') {

        $watch_rsync = 1;

    } elsif ($command eq 'rsync done') {

        if ($watch_rsync > 4) {

            $change_state = 'force';
            return 1;

        }

    } elsif ($command eq 'error') {

        $change_state = 'force';
        return 1;

    } elsif ($command eq 'force') {

        $change_state = 'force';
        return 1;

    } elsif ($command) {

        $change_state = 'force';
        output FATAL, "ERROR: change_state called with invalid command: $command\n";

    } elsif ($watch_rsync) {

        ++$watch_rsync;

    }

    return 0;

}

sub config
{
    my ($class, %args) = @_;
    
    my %config = ();
    
    my $config_fh = IO::File->new("< $option{config}")
        or output FATAL, "could not open config file $option{config}: $!\n";

    output DEBUG,  "\n* processing config $option{config} ";
    while (<$config_fh>) {
        chomp;                  # no newline
        s/#.*//;                # no comments
        s/^\s+//;               # no leading white
        s/\s+$//;               # no trailing white
        next unless length;     # anything left?
        my ($var, $value) = split(/\s*=\s*/, $_, 2);
        $config{ lc($var) } = $value;
        output DEBUG, '.';
    }

    $config_fh->close;
    output DEBUG, "done.\n";
    
    unless ($config{dont_die}) {
        output ERROR, "ERROR: you have not properly configured your config file.\n";
        output FATAL, "$0 will not run until you have done so.\n";
    }

    ## process local config file

    if (defined $config{config_local} and -e $config{config_local} ) {

        $config_fh = IO::File->new("< $config{config_local}")
            or output FATAL, "ERROR: could not open local config file $config{config_local} : $!\n";
    
        output DEBUG,  "* processing config $config{config_local} ";
        while (<$config_fh>) {
            chomp;                  # no newline
            s/#.*//;                # no comments
            s/^\s+//;               # no leading white
            s/\s+$//;               # no trailing white
            next unless length;     # anything left?
            my ($var, $value) = split(/\s*=\s*/, $_, 2);
            $config{ lc($var) } = $value;
            output DEBUG, '.';
        }

        $config_fh->close;
        output DEBUG, "done.\n";

        unless ($config{dont_die}) {
            output ERROR, "ERROR: you have not properly configured your local config file.\n";
            output FATAL, "$0 will not run until you have done so.\n";
        }

    }

    wantarray and return %config;
    return \%config;

}

1;
__END__
